// Demo 30: AimPracticeB.java
//
//   Simple Java GunSight Aiming Application.
//   Uses mouse to control gunsight.

//   Also illustrates dividing an Application Frame
//   into panels, subdividing a Panel into two other
//   Panels, and using a MousePadPanel and ControlPanel
//   to report events and display information  
//
//           by Charles W. Neville
//    Copyright Charles W. Neville, July, 1999

// This continues the MouseEvent and MousePadPanel approach to
// controlling an on screen object.  In this case, the object
// is a SimpleGunSight.

// Note: This is a more complex version of the AimPracticeA.java 
//       demo, with a much improved user interface.

import java.awt.*;
import java.awt.event.*;
import SimpleGunSight;
import MousePadPanelB;
import ControlPanelB;
import CloseAndExit;

public class AimPracticeB extends Frame
                          implements ActionListener, MouseListener,
                                     MouseMotionListener {

  // Transparent view Panel for spacing
  private Panel viewPanel;

  // Bottom Panel for TextFields, fire Button and mousepad
  private Panel bottomPanel;

  // ControlPanelB and MousePadPanelB to put in bottomPanel
  private ControlPanelB controlPanel;
  private MousePadPanelB mousePadPanel;
  
  private SimpleGunSight sight;

  private boolean firePressed = false;

  // Constructor: simply builds application Frame, which includes
  //              the several Panels and subPanels
  public AimPracticeB() {

     setLayout(new GridLayout(2, 1));  // two rows, one column

     viewPanel = new Panel();
     viewPanel.setSize(400, 200);
     viewPanel.setVisible(false);
     add(viewPanel);

     // build the bottomPanel
     bottomPanel = new Panel();
     bottomPanel.setLayout(new GridLayout(1, 2));  // one row,
                                                   // two columns
     bottomPanel.setSize(400, 200);

     controlPanel = new ControlPanelB(200, 200, this);
                         // note the "this".  WE process the control
                         // Panel's ActionEvents.
     bottomPanel.add(controlPanel); 
                          // this adds the whole controlPanel, including
                          // all its Labels, TextFields and Buttons.

     mousePadPanel = new MousePadPanelB(200, 200, this);
                          // note the "this".  WE process the control
                          // Panel's MouseEvents.
     bottomPanel.add(mousePadPanel);
                          // this adds the mousePadPanel
     add(bottomPanel);

     // call the Constructor for our GunSight class
     sight = new SimpleGunSight();

     addWindowListener(new CloseAndExit());

   } // Constructor AimPracticeA()

   // draw sight, maybe laser cannon tracer line,
   //   and dividing bar 
   public void paint(Graphics g) {

      g.setColor(Color.black);

      sight.drawSight(g);

      if(firePressed) {
        fireLaserCannon(g);
        firePressed = false;
      }

      g.fillRect(0, 200, 400, 10);


   } // paint()

   // fire laser cannon by drawing cannon tracer line,
   // sorry, no sound effects in deep space!
   private void fireLaserCannon(Graphics g) {

      g.setColor(Color.red);
      g.drawLine(200, 350, sight.getX(), sight.getY());
                                       // cannon tracer line
      g.setColor(Color.black);

   } // fireLaserCannon()

   // aim the sight in the x direction
   private void aimX(int x) {

     sight.setX(mousePadPanel.convertX(x));

   } // aimX()

   // aim the sight in the y direction
   private void aimY(int y) {

     sight.setY(y);

   } // aimY()
      
   // handle ActionEvents
   public void actionPerformed(ActionEvent e) {

      // the only ActionEvent is the one reported by
      // the controlPanel when its fireButton is pressed.

      firePressed = true;   
      repaint();

   } // actionPerformed

   // implement part of MouseListener interface

   // handle mouseClicked event
   public void mouseClicked (MouseEvent e) {

      // the only mouseClicked event is the one reported
      // by the mousePadPanel.  We will fire the laser
      // cannon then.

      firePressed = true;
      repaint();

   } // mouseClicked();

   // implement part of MouseMotionListener interface

   public void mouseDragged (MouseEvent e) {

      int x, y;

       x = e.getX();
       controlPanel.setXText(x);
                     // report the x coordinate in the
                     // xField of the controlPanel
       aimX(x);      // aim the sight

       y = e.getY();
       controlPanel.setYText(y);
                     // report the y coordinate in the
                     // yField of the controlPanel
       aimY(y);      // aim the sight

       repaint();

   } // mouseDragged();

   // unused methods from the MouseListener interface
   //   -- they have to be here to implement it.

   public void mousePressed (MouseEvent e) {
   }

   public void mouseReleased (MouseEvent e) {
   }

   public void mouseEntered (MouseEvent e) {
   }

   public void mouseExited (MouseEvent e) {
   }

   // unused method from the MouseMotionListener
   //   interface -- has to be here to implement it.

   public void mouseMoved (MouseEvent e) {
   }

   public static void main(String[] args) {

      AimPracticeB apb = new AimPracticeB();  
      apb.setSize(400, 400);  // set our size
      apb.setVisible(true);   // make ourselves visible

   } // main()

} // class AimPracticeA
